@@ -16,13 +16,16 @@
 from __future__ import absolute_import
 import itertools
 import operator
+import os.path
+import uuid
 
 from oslo_db import exception
 from oslo_db.sqlalchemy import models
 from oslo_db.sqlalchemy import session
-from oslo_utils import timeutils
+from oslo_db.sqlalchemy import utils as oslo_db_utils
 import six
 import sqlalchemy
+import sqlalchemy_utils
 from stevedore import extension
 
 from gnocchi import exceptions
@@ -45,14 +48,15 @@ def get_resource_mappers(ext):
         resource_ext = ext.plugin
         resource_history_ext = ResourceHistory
     else:
+        tablename = getattr(ext.plugin, '__tablename__', ext.name)
         resource_ext = type(str(ext.name),
                             (ext.plugin, base.ResourceExtMixin, Resource),
-                            {"__tablename__": ext.name})
+                            {"__tablename__": tablename})
         resource_history_ext = type(str("%s_history" % ext.name),
                                     (ext.plugin, base.ResourceHistoryExtMixin,
                                      ResourceHistory),
                                     {"__tablename__": (
-                                        "%s_history" % ext.name)})
+                                        "%s_history" % tablename)})
 
     return {'resource': resource_ext,
             'history': resource_history_ext}
@@ -74,9 +78,32 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
     def disconnect(self):
         self.engine_facade.get_engine().dispose()
 
-    def upgrade(self):
-        engine = self.engine_facade.get_engine()
-        Base.metadata.create_all(engine, checkfirst=True)
+    def _get_alembic_config(self):
+        from alembic import config
+
+        cfg = config.Config(
+            "%s/alembic/alembic.ini" % os.path.dirname(__file__))
+        cfg.set_main_option('sqlalchemy.url',
+                            self.conf.database.connection)
+        return cfg
+
+    def upgrade(self, nocreate=False):
+        from alembic import command
+        from alembic import migration
+
+        cfg = self._get_alembic_config()
+        cfg.conf = self.conf
+        if nocreate:
+            command.upgrade(cfg, "head")
+        else:
+            engine = self.engine_facade.get_engine()
+            ctxt = migration.MigrationContext.configure(engine.connect())
+            current_version = ctxt.get_current_revision()
+            if current_version is None:
+                Base.metadata.create_all(engine)
+                command.stamp(cfg, "head")
+            else:
+                command.upgrade(cfg, "head")
 
     def _resource_type_to_class(self, resource_type, purpose="resource"):
         if resource_type not in self._RESOURCE_CLASS_MAPPER:
@@ -85,11 +112,15 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
 
     def list_archive_policies(self):
         session = self.engine_facade.get_session()
-        return session.query(ArchivePolicy).all()
+        aps = list(session.query(ArchivePolicy).all())
+        session.expunge_all()
+        return aps
 
     def get_archive_policy(self, name):
         session = self.engine_facade.get_session()
-        return session.query(ArchivePolicy).get(name)
+        ap = session.query(ArchivePolicy).get(name)
+        session.expunge_all()
+        return ap
 
     def delete_archive_policy(self, name):
         session = self.engine_facade.get_session()
@@ -97,22 +128,22 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
             if session.query(ArchivePolicy).filter(
                     ArchivePolicy.name == name).delete() == 0:
                 raise indexer.NoSuchArchivePolicy(name)
-        except exception.DBError as e:
-            # TODO(jd) Add an exception in oslo.db to match foreign key
-            # violations
-            if isinstance(e.inner_exception, sqlalchemy.exc.IntegrityError):
+        except exception.DBReferenceError as e:
+            if (e.constraint ==
+               'fk_metric_archive_policy_name_archive_policy_name'):
                 raise indexer.ArchivePolicyInUse(name)
+            raise
 
     def get_metrics(self, uuids):
         if not uuids:
             return []
         session = self.engine_facade.get_session()
         query = session.query(Metric).filter(Metric.id.in_(uuids)).options(
-            sqlalchemy.orm.joinedload(
-                Metric.archive_policy)).options(
-                    sqlalchemy.orm.joinedload(Metric.resource))
+            sqlalchemy.orm.joinedload('resource'))
 
-        return list(query.all())
+        metrics = list(query.all())
+        session.expunge_all()
+        return metrics
 
     def create_archive_policy(self, archive_policy):
         ap = ArchivePolicy(
@@ -127,26 +158,26 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
             session.flush()
         except exception.DBDuplicateEntry:
             raise indexer.ArchivePolicyAlreadyExists(archive_policy.name)
+        session.expunge_all()
         return ap
 
     def list_archive_policy_rules(self):
         session = self.engine_facade.get_session()
-        return session.query(ArchivePolicyRule).all()
+        aps = session.query(ArchivePolicyRule).all()
+        session.expunge_all()
+        return aps
 
     def get_archive_policy_rule(self, name):
         session = self.engine_facade.get_session()
-        return session.query(ArchivePolicyRule).get(name)
+        ap = session.query(ArchivePolicyRule).get(name)
+        session.expunge_all()
+        return ap
 
     def delete_archive_policy_rule(self, name):
         session = self.engine_facade.get_session()
-        try:
-            if session.query(ArchivePolicyRule).filter(
-                    ArchivePolicyRule.name == name).delete() == 0:
-                raise indexer.NoSuchArchivePolicyRule(name)
-        except exception.DBError as e:
-            # TODO(prad): fix foreign key violations when oslo.db supports it
-            if isinstance(e.inner_exception, sqlalchemy.exc.IntegrityError):
-                raise indexer.ArchivePolicyRuleInUse(name)
+        if session.query(ArchivePolicyRule).filter(
+                ArchivePolicyRule.name == name).delete() == 0:
+            raise indexer.NoSuchArchivePolicyRule(name)
 
     def create_archive_policy_rule(self, name, metric_pattern,
                                    archive_policy_name):
@@ -161,12 +192,12 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
             session.flush()
         except exception.DBDuplicateEntry:
             raise indexer.ArchivePolicyRuleAlreadyExists(name)
+        session.expunge_all()
         return apr
 
     def create_metric(self, id, created_by_user_id, created_by_project_id,
                       archive_policy_name,
-                      name=None, resource_id=None,
-                      details=False):
+                      name=None, resource_id=None):
         m = Metric(id=id,
                    created_by_user_id=created_by_user_id,
                    created_by_project_id=created_by_project_id,
@@ -175,20 +206,32 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
                    resource_id=resource_id)
         session = self.engine_facade.get_session()
         session.add(m)
-        session.flush()
-        if details:
-            # Fetch archive policy
-            m.archive_policy
+        try:
+            session.flush()
+        except exception.DBReferenceError as e:
+            if (e.constraint ==
+               'fk_metric_archive_policy_name_archive_policy_name'):
+                raise indexer.NoSuchArchivePolicy(archive_policy_name)
+            raise
+        session.expunge_all()
         return m
 
-    def list_metrics(self, user_id=None, project_id=None):
+    def list_metrics(self, user_id=None, project_id=None, details=False,
+                     **kwargs):
         session = self.engine_facade.get_session()
         q = session.query(Metric)
         if user_id is not None:
             q = q.filter(Metric.created_by_user_id == user_id)
         if project_id is not None:
             q = q.filter(Metric.created_by_project_id == project_id)
-        return q.all()
+        for attr in kwargs:
+            q = q.filter(getattr(Metric, attr) == kwargs[attr])
+        if details:
+            q = q.options(sqlalchemy.orm.joinedload('resource'))
+
+        metrics = list(q.all())
+        session.expunge_all()
+        return metrics
 
     def create_resource(self, resource_type, id,
                         created_by_user_id, created_by_project_id,
@@ -227,6 +270,7 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
         # NOTE(jd) Force load of metrics :)
         r.metrics
 
+        session.expunge_all()
         return r
 
     def update_resource(self, resource_type,
@@ -234,77 +278,106 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
                         append_metrics=False,
                         **kwargs):
 
-        now = timeutils.utcnow()
+        now = utils.utcnow()
 
         resource_cls = self._resource_type_to_class(resource_type)
         resource_history_cls = self._resource_type_to_class(resource_type,
                                                             "history")
         session = self.engine_facade.get_session()
-        with session.begin():
-            # NOTE(sileht): We use FOR UPDATE that is not galera friendly,
-            # but they are no other way to cleanly patch a resource and store
-            # the history that safe when two concurrent calls are done.
-            q = session.query(resource_cls).filter(
-                resource_cls.id == resource_id).with_for_update()
+        try:
+            with session.begin():
+                # NOTE(sileht): We use FOR UPDATE that is not galera friendly,
+                # but they are no other way to cleanly patch a resource and
+                # store the history that safe when two concurrent calls are
+                # done.
+                q = session.query(resource_cls).filter(
+                    resource_cls.id == resource_id).with_for_update()
 
-            r = q.first()
-            if r is None:
-                raise indexer.NoSuchResource(resource_id)
+                r = q.first()
+                if r is None:
+                    raise indexer.NoSuchResource(resource_id)
 
-            # Build history
-            rh = resource_history_cls()
-            for col in sqlalchemy.inspect(resource_cls).columns:
-                setattr(rh, col.name, getattr(r, col.name))
-            rh.revision_end = now
-            session.add(rh)
+                # Build history
+                rh = resource_history_cls()
+                for col in sqlalchemy.inspect(resource_cls).columns:
+                    setattr(rh, col.name, getattr(r, col.name))
+                rh.revision_end = now
+                session.add(rh)
 
-            # Update the resource
-            if ended_at is not _marker:
-                # NOTE(jd) Could be better to have check in the db for that so
-                # we can just run the UPDATE
-                if r.started_at is not None and ended_at is not None:
-                    # Convert to UTC because we store in UTC :(
-                    ended_at = timeutils.normalize_time(ended_at)
-                    if r.started_at > ended_at:
-                        raise ValueError(
-                            "Start timestamp cannot be after end timestamp")
-                r.ended_at = ended_at
+                # Update the resource
+                if ended_at is not _marker:
+                    # NOTE(jd) MySQL does not honor checks. I hate it.
+                    engine = self.engine_facade.get_engine()
+                    if engine.dialect.name == "mysql":
+                        if r.started_at is not None and ended_at is not None:
+                            if r.started_at > ended_at:
+                                raise indexer.ResourceValueError(
+                                    resource_type, "ended_at", ended_at)
+                    r.ended_at = ended_at
 
-            r.revision_start = now
+                r.revision_start = now
 
-            if kwargs:
-                for attribute, value in six.iteritems(kwargs):
-                    if hasattr(r, attribute):
-                        setattr(r, attribute, value)
-                    else:
-                        raise indexer.ResourceAttributeError(
-                            r.type, attribute)
+                if kwargs:
+                    for attribute, value in six.iteritems(kwargs):
+                        if hasattr(r, attribute):
+                            setattr(r, attribute, value)
+                        else:
+                            raise indexer.ResourceAttributeError(
+                                r.type, attribute)
 
-            if metrics is not _marker:
-                if not append_metrics:
-                    session.query(Metric).filter(
-                        Metric.resource_id == resource_id).update(
-                            {"resource_id": None})
-                self._set_metrics_for_resource(session, r, metrics)
+                if metrics is not _marker:
+                    if not append_metrics:
+                        session.query(Metric).filter(
+                            Metric.resource_id == resource_id).update(
+                                {"resource_id": None})
+                    self._set_metrics_for_resource(session, r, metrics)
+        except exception.DBConstraintError as e:
+            if e.check_name == "ck_started_before_ended":
+                raise indexer.ResourceValueError(
+                    resource_type, "ended_at", ended_at)
+            raise
 
         # NOTE(jd) Force load of metrics – do it outside the session!
         r.metrics
 
+        session.expunge_all()
         return r
 
     @staticmethod
     def _set_metrics_for_resource(session, r, metrics):
-        for name, metric_id in six.iteritems(metrics):
-            try:
-                update = session.query(Metric).filter(
-                    Metric.id == metric_id,
-                    Metric.created_by_user_id == r.created_by_user_id,
-                    Metric.created_by_project_id == r.created_by_project_id,
-                ).update({"resource_id": r.id, "name": name})
-            except exception.DBDuplicateEntry:
-                raise indexer.NamedMetricAlreadyExists(name)
-            if update == 0:
-                raise indexer.NoSuchMetric(metric_id)
+        for name, value in six.iteritems(metrics):
+            if isinstance(value, uuid.UUID):
+                try:
+                    update = session.query(Metric).filter(
+                        Metric.id == value,
+                        (Metric.created_by_user_id
+                         == r.created_by_user_id),
+                        (Metric.created_by_project_id
+                         == r.created_by_project_id),
+                    ).update({"resource_id": r.id, "name": name})
+                except exception.DBDuplicateEntry:
+                    raise indexer.NamedMetricAlreadyExists(name)
+                if update == 0:
+                    raise indexer.NoSuchMetric(value)
+            else:
+                ap_name = value['archive_policy_name']
+                m = Metric(id=uuid.uuid4(),
+                           created_by_user_id=r.created_by_user_id,
+                           created_by_project_id=r.created_by_project_id,
+                           archive_policy_name=ap_name,
+                           name=name,
+                           resource_id=r.id)
+                session.add(m)
+                try:
+                    session.flush()
+                except exception.DBDuplicateEntry:
+                    raise indexer.NamedMetricAlreadyExists(name)
+                except exception.DBReferenceError as e:
+                    if (e.constraint ==
+                       'fk_metric_archive_policy_name_archive_policy_name'):
+                        raise indexer.NoSuchArchivePolicy(ap_name)
+                    raise
+
         session.expire(r, ['metrics'])
 
     def delete_resource(self, resource_id, delete_metrics=None):
@@ -312,7 +385,7 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
         with session.begin():
             q = session.query(Resource).filter(
                 Resource.id == resource_id).options(
-                    sqlalchemy.orm.joinedload(Resource.metrics))
+                    sqlalchemy.orm.joinedload('metrics'))
             r = q.first()
             if r is None:
                 raise indexer.NoSuchResource(resource_id)
@@ -327,8 +400,10 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
             resource_cls).filter(
                 resource_cls.id == resource_id)
         if with_metrics:
-            q = q.options(sqlalchemy.orm.joinedload(resource_cls.metrics))
-        return q.first()
+            q = q.options(sqlalchemy.orm.joinedload('metrics'))
+        r = q.first()
+        session.expunge_all()
+        return r
 
     def _get_history_result_mapper(self, resource_type):
         resource_cls = self._resource_type_to_class(resource_type)
@@ -370,7 +445,11 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
     def list_resources(self, resource_type='generic',
                        attribute_filter=None,
                        details=False,
-                       history=False):
+                       history=False,
+                       limit=None,
+                       marker=None,
+                       sorts=None):
+        sorts = sorts or []
 
         session = self.engine_facade.get_session()
 
@@ -395,9 +474,37 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
 
             q = q.filter(f)
 
+        # transform the api-wg representation to the oslo.db one
+        sort_keys = []
+        sort_dirs = []
+        for sort in sorts:
+            sort_key, __, sort_dir = sort.partition(":")
+            sort_keys.append(sort_key.strip())
+            sort_dirs.append(sort_dir or 'asc')
+
+        # paginate_query require at list one uniq column
+        if 'id' not in sort_keys:
+            sort_keys.append('id')
+            sort_dirs.append('asc')
+
+        if marker:
+            resource_marker = self.get_resource(resource_type, marker)
+            if resource_marker is None:
+                raise indexer.InvalidPagination(
+                    "Invalid marker: `%s'" % marker)
+        else:
+            resource_marker = None
+
+        try:
+            q = oslo_db_utils.paginate_query(q, target_cls, limit=limit,
+                                             sort_keys=sort_keys,
+                                             marker=resource_marker,
+                                             sort_dirs=sort_dirs)
+        except (exception.InvalidSortKey, ValueError) as e:
+            raise indexer.InvalidPagination(e)
+
         # Always include metrics
         q = q.options(sqlalchemy.orm.joinedload("metrics"))
-        q = q.order_by(target_cls.revision_start)
         all_resources = q.all()
 
         if details:
@@ -420,15 +527,15 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
 
                     q = session.query(target_cls).filter(f)
                     # Always include metrics
-                    q = q.options(
-                        sqlalchemy.orm.joinedload(target_cls.metrics))
+                    q = q.options(sqlalchemy.orm.joinedload('metrics'))
                     all_resources.extend(q.all())
+        session.expunge_all()
         return all_resources
 
     def delete_metric(self, id):
         session = self.engine_facade.get_session()
-        session.query(Metric).filter(Metric.id == id).delete()
-        session.flush()
+        if session.query(Metric).filter(Metric.id == id).delete() == 0:
+            raise indexer.NoSuchMetric(id)
 
 
 class QueryTransformer(object):
@@ -509,9 +616,20 @@ class QueryTransformer(object):
                 raise indexer.QueryAttributeError(table, field_name)
 
             # Convert value to the right type
-            if value is not None and isinstance(attr.type,
-                                                base.PreciseTimestamp):
-                value = utils.to_timestamp(value)
+            if value is not None:
+                converter = None
+
+                if isinstance(attr.type, base.PreciseTimestamp):
+                    converter = utils.to_timestamp
+                elif (isinstance(attr.type, sqlalchemy_utils.UUIDType)
+                      and not isinstance(value, uuid.UUID)):
+                    converter = utils.ResourceUUID
+
+                if converter:
+                    try:
+                        value = converter(value)
+                    except Exception:
+                        raise indexer.QueryValueError(value, field_name)
 
         return op(attr, value)
 
