@@ -20,6 +20,7 @@ import calendar
 import datetime
 import decimal
 
+import iso8601
 from oslo_db.sqlalchemy import models
 from oslo_utils import timeutils
 from oslo_utils import units
@@ -32,6 +33,7 @@ import sqlalchemy_utils
 from gnocchi import archive_policy
 from gnocchi import indexer
 from gnocchi import storage
+from gnocchi import utils
 
 Base = declarative.declarative_base()
 
@@ -76,17 +78,26 @@ class PreciseTimestamp(types.TypeDecorator):
                 types.DECIMAL(precision=20,
                               scale=6,
                               asdecimal=True))
-        return self.impl
+        return dialect.type_descriptor(self.impl)
+
+    def compare_against_backend(self, dialect, conn_type):
+        if dialect.name == 'mysql':
+            return issubclass(type(conn_type), types.DECIMAL)
+        return issubclass(type(conn_type), type(self.impl))
 
     def process_bind_param(self, value, dialect):
+        if value is not None:
+            value = timeutils.normalize_time(value)
         if dialect.name == 'mysql':
             return self._dt_to_decimal(value)
         return value
 
     def process_result_value(self, value, dialect):
         if dialect.name == 'mysql':
-            return self._decimal_to_dt(value)
-        return value
+            value = self._decimal_to_dt(value)
+        if value is not None:
+            return timeutils.normalize_time(value).replace(
+                tzinfo=iso8601.iso8601.UTC)
 
 
 class GnocchiBase(models.ModelBase):
@@ -130,7 +141,7 @@ class Metric(Base, GnocchiBase, storage.Metric):
         COMMON_TABLES_ARGS,
     )
 
-    id = sqlalchemy.Column(sqlalchemy_utils.UUIDType(binary=False),
+    id = sqlalchemy.Column(sqlalchemy_utils.UUIDType(),
                            primary_key=True)
     archive_policy_name = sqlalchemy.Column(
         sqlalchemy.String(255),
@@ -139,13 +150,13 @@ class Metric(Base, GnocchiBase, storage.Metric):
             ondelete="RESTRICT",
             name="fk_metric_archive_policy_name_archive_policy_name"),
         nullable=False)
-    archive_policy = sqlalchemy.orm.relationship(ArchivePolicy)
+    archive_policy = sqlalchemy.orm.relationship(ArchivePolicy, lazy="joined")
     created_by_user_id = sqlalchemy.Column(
-        sqlalchemy_utils.UUIDType(binary=False))
+        sqlalchemy_utils.UUIDType())
     created_by_project_id = sqlalchemy.Column(
-        sqlalchemy_utils.UUIDType(binary=False))
+        sqlalchemy_utils.UUIDType())
     resource_id = sqlalchemy.Column(
-        sqlalchemy_utils.UUIDType(binary=False),
+        sqlalchemy_utils.UUIDType(),
         sqlalchemy.ForeignKey('resource.id',
                               ondelete="CASCADE",
                               name="fk_metric_resource_id_resource_id"))
@@ -159,14 +170,14 @@ class Metric(Base, GnocchiBase, storage.Metric):
             "name": self.name,
         }
         unloaded = sqlalchemy.inspect(self).unloaded
-        if 'archive_policy' in unloaded:
-            d['archive_policy_name'] = self.archive_policy_name
-        else:
-            d['archive_policy'] = self.archive_policy
         if 'resource' in unloaded:
             d['resource_id'] = self.resource_id
         else:
             d['resource'] = self.resource
+        if 'archive_policy' in unloaded:
+            d['archive_policy_name'] = self.archive_policy_name
+        else:
+            d['archive_policy'] = self.archive_policy
         return d
 
     def __eq__(self, other):
@@ -183,13 +194,15 @@ class Metric(Base, GnocchiBase, storage.Metric):
                  and self.resource_id == other.resource_id)
                 or (storage.Metric.__eq__(self, other)))
 
+    __hash__ = storage.Metric.__hash__
+
 
 class ResourceJsonifier(indexer.Resource):
     def jsonify(self):
         d = dict(self)
         del d['revision']
         if 'metrics' not in sqlalchemy.inspect(self).unloaded:
-            d['metrics'] = dict((m['name'], six.text_type(m['id']))
+            d['metrics'] = dict((m.name, six.text_type(m.id))
                                 for m in self.metrics)
         return d
 
@@ -198,19 +211,22 @@ class ResourceMixin(ResourceJsonifier):
     @declarative.declared_attr
     def __table_args__(cls):
         return (sqlalchemy.Index('ix_%s_id' % cls.__tablename__, 'id'),
+                sqlalchemy.CheckConstraint('started_at <= ended_at',
+                                           name="ck_started_before_ended"),
                 COMMON_TABLES_ARGS)
 
     type = sqlalchemy.Column(sqlalchemy.Enum('generic', 'instance',
                                              'swift_account', 'volume',
                                              'ceph_account', 'network',
                                              'identity', 'ipmi', 'stack',
-                                             'image',
+                                             'image', 'instance_disk',
+                                             'instance_network_interface',
                                              name="resource_type_enum"),
                              nullable=False, default='generic')
     created_by_user_id = sqlalchemy.Column(
-        sqlalchemy_utils.UUIDType(binary=False))
+        sqlalchemy_utils.UUIDType())
     created_by_project_id = sqlalchemy.Column(
-        sqlalchemy_utils.UUIDType(binary=False))
+        sqlalchemy_utils.UUIDType())
     started_at = sqlalchemy.Column(PreciseTimestamp, nullable=False,
                                    # NOTE(jd): We would like to use
                                    # sqlalchemy.func.now, but we can't
@@ -218,19 +234,19 @@ class ResourceMixin(ResourceJsonifier):
                                    # MySQL is not a Timestamp, so it would
                                    # not store a timestamp but a date as an
                                    # integer.
-                                   default=datetime.datetime.utcnow)
+                                   default=lambda: utils.utcnow())
     revision_start = sqlalchemy.Column(PreciseTimestamp, nullable=False,
-                                       default=timeutils.utcnow)
+                                       default=lambda: utils.utcnow())
     ended_at = sqlalchemy.Column(PreciseTimestamp)
-    user_id = sqlalchemy.Column(sqlalchemy_utils.UUIDType(binary=False))
-    project_id = sqlalchemy.Column(sqlalchemy_utils.UUIDType(binary=False))
+    user_id = sqlalchemy.Column(sqlalchemy_utils.UUIDType())
+    project_id = sqlalchemy.Column(sqlalchemy_utils.UUIDType())
 
 
 class Resource(ResourceMixin, Base, GnocchiBase):
     __tablename__ = 'resource'
     _extra_keys = ['revision', 'revision_end']
     revision = -1
-    id = sqlalchemy.Column(sqlalchemy_utils.UUIDType(binary=False),
+    id = sqlalchemy.Column(sqlalchemy_utils.UUIDType(),
                            primary_key=True)
     revision_end = None
     metrics = sqlalchemy.orm.relationship(Metric, backref="resource")
@@ -249,14 +265,14 @@ class ResourceHistory(ResourceMixin, Base, GnocchiBase):
     __tablename__ = 'resource_history'
     revision = sqlalchemy.Column(sqlalchemy.Integer, autoincrement=True,
                                  primary_key=True)
-    id = sqlalchemy.Column(sqlalchemy_utils.UUIDType(binary=False),
+    id = sqlalchemy.Column(sqlalchemy_utils.UUIDType(),
                            sqlalchemy.ForeignKey(
                                'resource.id',
                                ondelete="CASCADE",
                                name="fk_resource_history_id_resource_id"),
                            nullable=False)
     revision_end = sqlalchemy.Column(PreciseTimestamp, nullable=False,
-                                     default=timeutils.utcnow)
+                                     default=lambda: utils.utcnow())
     metrics = sqlalchemy.orm.relationship(
         Metric, primaryjoin="Metric.resource_id == ResourceHistory.id",
         foreign_keys='Metric.resource_id')
@@ -278,7 +294,7 @@ class ResourceExtMixin(object):
     @declarative.declared_attr
     def id(cls):
         return sqlalchemy.Column(
-            sqlalchemy_utils.UUIDType(binary=False),
+            sqlalchemy_utils.UUIDType(),
             sqlalchemy.ForeignKey(
                 'resource.id',
                 ondelete="CASCADE",
