@@ -20,12 +20,12 @@ try:
 except ImportError:
     import trollius as asyncio
 from oslo_log import log
-from oslo_utils import timeutils
 import six
 
 from gnocchi import indexer
 from gnocchi import service
 from gnocchi import storage
+from gnocchi import utils
 
 
 LOG = log.getLogger(__name__)
@@ -64,14 +64,14 @@ class Stats(object):
                     "Invalid sampling for ms: `%d`, should be none"
                     % sampling)
             self.times[metric_name] = storage.Measure(
-                timeutils.utcnow(), value)
+                utils.utcnow(), value)
         elif metric_type == "g":
             if sampling is not None:
                 raise ValueError(
                     "Invalid sampling for g: `%d`, should be none"
                     % sampling)
             self.gauges[metric_name] = storage.Measure(
-                timeutils.utcnow(), value)
+                utils.utcnow(), value)
         elif metric_type == "c":
             sampling = 1 if sampling is None else sampling
             if metric_name in self.counters:
@@ -79,7 +79,7 @@ class Stats(object):
             else:
                 current_value = 0
             self.counters[metric_name] = storage.Measure(
-                timeutils.utcnow(),
+                utils.utcnow(),
                 current_value + (value * (1 / sampling)))
         # TODO(jd) Support "set" type
         # elif metric_type == "s":
@@ -110,12 +110,7 @@ class Stats(object):
                         self.conf.statsd.project_id,
                         archive_policy_name=ap_name,
                         name=metric_name,
-                        resource_id=self.conf.statsd.resource_id,
-                        details=True)
-                    # TODO(jd) Slight optimization would be to change the
-                    # driver to allow creation of metric with initial measures
-                    # included!
-                    self.storage.create_metric(metric)
+                        resource_id=self.conf.statsd.resource_id)
                 self.storage.add_measures(metric, (measure,))
             except Exception as e:
                 LOG.error("Unable to add measure %s: %s"
