@@ -13,9 +13,9 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
-import collections
-
+import operator
 from oslo_config import cfg
+from oslo_utils import timeutils
 from stevedore import driver
 
 from gnocchi import exceptions
@@ -25,10 +25,22 @@ OPTS = [
     cfg.StrOpt('driver',
                default='file',
                help='Storage driver to use'),
+    cfg.IntOpt('metric_processing_delay',
+               default=5,
+               help="How many seconds to wait between "
+               "new metric measure processing"),
 ]
 
 
-Measure = collections.namedtuple('Measure', ['timestamp', 'value'])
+class Measure(object):
+    def __init__(self, timestamp, value):
+        self.timestamp = timeutils.normalize_time(timestamp)
+        self.value = value
+
+    def __iter__(self):
+        """Allow to transform measure to tuple."""
+        yield self.timestamp
+        yield self.value
 
 
 class Metric(object):
@@ -47,9 +59,6 @@ class Metric(object):
     def __repr__(self):
         return '<%s %s>' % (self.__class__.__name__, self.id)
 
-    def __hash__(self):
-        return id(self)
-
     def __str__(self):
         return str(self.id)
 
@@ -62,6 +71,8 @@ class Metric(object):
                 and self.name == other.name
                 and self.resource_id == other.resource_id)
 
+    __hash__ = object.__hash__
+
 
 class InvalidQuery(Exception):
     pass
@@ -96,16 +107,6 @@ class MetricAlreadyExists(Exception):
             "Metric %s already exists" % metric)
 
 
-class NoDeloreanAvailable(Exception):
-    """Error raised when trying to insert a value that is too old."""
-
-    def __init__(self, first_timestamp, bad_timestamp):
-        self.first_timestamp = first_timestamp
-        self.bad_timestamp = bad_timestamp
-        super(NoDeloreanAvailable, self).__init__(
-            "%s is before %s" % (bad_timestamp, first_timestamp))
-
-
 class MetricUnaggregatable(Exception):
     """Error raised when metrics can't be aggregated."""
 
@@ -117,21 +118,18 @@ class MetricUnaggregatable(Exception):
             % (" ,".join((str(m.id) for m in metrics)), reason))
 
 
-def _get_driver(name, conf):
-    """Return the driver named name.
+def get_driver_class(conf):
+    """Return the storage driver class.
 
-    :param name: The name of the driver.
-    :param conf: The conf to pass to the driver.
+    :param conf: The conf to use to determine the driver.
     """
-    d = driver.DriverManager('gnocchi.storage',
-                             name).driver
-    return d(conf)
+    return driver.DriverManager('gnocchi.storage',
+                                conf.storage.driver).driver
 
 
 def get_driver(conf):
     """Return the configured driver."""
-    return _get_driver(conf.storage.driver,
-                       conf.storage)
+    return get_driver_class(conf)(conf.storage)
 
 
 class StorageDriver(object):
@@ -140,12 +138,8 @@ class StorageDriver(object):
         pass
 
     @staticmethod
-    def create_metric(metric):
-        """Create a metric.
-
-        :param metric: The metric object.
-        """
-        raise exceptions.NotImplementedError
+    def stop():
+        pass
 
     @staticmethod
     def add_measures(metric, measures):
@@ -156,6 +150,14 @@ class StorageDriver(object):
         """
         raise exceptions.NotImplementedError
 
+    @staticmethod
+    def process_measures(indexer=None):
+        """Process added measures in background.
+
+        Some drivers might need to have a background task running that process
+        the measures sent to metrics. This is used for that.
+        """
+
     @staticmethod
     def get_measures(metric, from_timestamp=None, to_timestamp=None,
                      aggregation='mean'):
@@ -166,7 +168,8 @@ class StorageDriver(object):
         :param to timestamp: The timestamp to get the measure to.
         :param aggregation: The type of aggregation to retrieve.
         """
-        raise exceptions.NotImplementedError
+        if aggregation not in metric.archive_policy.aggregation_methods:
+            raise AggregationDoesNotExist(metric, aggregation)
 
     @staticmethod
     def delete_metric(metric):
@@ -183,7 +186,9 @@ class StorageDriver(object):
         :param to timestamp: The timestamp to get the measure to.
         :param aggregation: The type of aggregation to retrieve.
         """
-        raise exceptions.NotImplementedError
+        for metric in metrics:
+            if aggregation not in metric.archive_policy.aggregation_methods:
+                raise AggregationDoesNotExist(metric, aggregation)
 
     @staticmethod
     def search_value(metrics, query, from_timestamp=None,
@@ -198,3 +203,96 @@ class StorageDriver(object):
         :param aggregation: The type of aggregation to retrieve.
         """
         raise exceptions.NotImplementedError
+
+
+class MeasureQuery(object):
+    binary_operators = {
+        u"=": operator.eq,
+        u"==": operator.eq,
+        u"eq": operator.eq,
+
+        u"<": operator.lt,
+        u"lt": operator.lt,
+
+        u">": operator.gt,
+        u"gt": operator.gt,
+
+        u"<=": operator.le,
+        u"≤": operator.le,
+        u"le": operator.le,
+
+        u">=": operator.ge,
+        u"≥": operator.ge,
+        u"ge": operator.ge,
+
+        u"!=": operator.ne,
+        u"≠": operator.ne,
+        u"ne": operator.ne,
+
+        u"%": operator.mod,
+        u"mod": operator.mod,
+
+        u"+": operator.add,
+        u"add": operator.add,
+
+        u"-": operator.sub,
+        u"sub": operator.sub,
+
+        u"*": operator.mul,
+        u"×": operator.mul,
+        u"mul": operator.mul,
+
+        u"/": operator.truediv,
+        u"÷": operator.truediv,
+        u"div": operator.truediv,
+
+        u"**": operator.pow,
+        u"^": operator.pow,
+        u"pow": operator.pow,
+    }
+
+    multiple_operators = {
+        u"or": any,
+        u"∨": any,
+        u"and": all,
+        u"∧": all,
+    }
+
+    def __init__(self, tree):
+        self._eval = self.build_evaluator(tree)
+
+    def __call__(self, value):
+        return self._eval(value)
+
+    def build_evaluator(self, tree):
+        try:
+            operator, nodes = list(tree.items())[0]
+        except Exception:
+            return lambda value: tree
+        try:
+            op = self.multiple_operators[operator]
+        except KeyError:
+            try:
+                op = self.binary_operators[operator]
+            except KeyError:
+                raise InvalidQuery("Unknown operator %s" % operator)
+            return self._handle_binary_op(op, nodes)
+        return self._handle_multiple_op(op, nodes)
+
+    def _handle_multiple_op(self, op, nodes):
+        elements = [self.build_evaluator(node) for node in nodes]
+        return lambda value: op((e(value) for e in elements))
+
+    def _handle_binary_op(self, op, node):
+        try:
+            iterator = iter(node)
+        except Exception:
+            return lambda value: op(value, node)
+        nodes = list(iterator)
+        if len(nodes) != 2:
+            raise InvalidQuery(
+                "Binary operator %s needs 2 arguments, %d given" %
+                (op, len(nodes)))
+        node0 = self.build_evaluator(node[0])
+        node1 = self.build_evaluator(node[1])
+        return lambda value: op(node0(value), node1(value))
